<?php

require_once "d2p_manager.inc";

class	d2p_data
{
	var	$objId;
	var	$manager;
	var	$dbConn;
	var	$table;
	var	$identifierName;
	var	$fields;
	var	$modified;
	var	$state;
	var	$errorMsg;

	function	d2p_data(&$manager, $table, $identifierName)
	{
		$this->objId = uniqid("d2p_data-", true);
		$this->manager = &$manager;
		$this->table = $table;
		$this->modified = false;
		$this->dbConn = $this->manager->getDatabaseConnection();
		$this->identifierName = $identifierName;
		$this->state = "n";
		$this->errorMsg = "";
		$this->prepareFields();
	}

	function	prepareFields()
	{
		$this->fields = array();
	}

	function	loadDatabaseEntry($values)
	{
		if	(!is_array($values))
			return	false;

		$this->state = "m";
		foreach ($this->fields as $fieldName => $details)
		{
			$this->fields[$fieldName]["initial_value"] = $this->fields[$fieldName]["current_value"] =
				$values[$fieldName];
		}

		return	true;
	}

	function	destroy()
	{
		if	($this->state != "m")
			return	false;
		$this->state = "s";
		$this->modified = true;
		$this->manager->objectModified($this);
	}

	function	getField($fieldName)
	{
		if	(is_array($this->fields[$fieldName]))
		{
			if	($this->fields[$fieldName]["references"])
				return	$this->manager->objectFromBase(
						$this->fields[$fieldName]["references"],
						$this->fields[$fieldName]["current_value"]
					);
			else
				return	$this->fields[$fieldName]["current_value"];
		}
		return	NULL;
	}

	function	setField($fieldName, $value)
	{
		if	(is_array($this->fields[$fieldName]) && $this->state != "s")
		{
			if	(is_null($value) && $this->fields[$fieldName]["mandatory"])
				return	false;

			if	($this->fields[$fieldName]["references"])
				$this->fields[$fieldName]["current_value"] = $value->getIdentifier();
			else
				$this->fields[$fieldName]["current_value"] = $value;

			$this->modified = true;
			$this->manager->objectModified($this);
			return	true;
		}
		return	false;
	}

	function	fieldToSQL($type, $value)
	{
		if	(is_null($value))
			return	"NULL";
		elseif	(preg_match("/.*int$/", $type))
			return	$value;
		elseif	($type == "bool")
			return	($value ? "true" : "false");
		else
			return	"'" . preg_replace("/\\'/", "''", $value) . "'";
	}

	function	getIdentifierName()
	{
		return	$this->identifierName;
	}

	function	getIdentifier()
	{
		return	$this->getField($this->identifierName);
	}

	function	getDescription()
	{
		return	$this->getIdentifier();
	}

	function	getHTMLForm()
	{
		return	"";
	}

	function	createRequest()
	{
		$request = "";

		if	($this->state == "m" && $this->modified)
		{
			$request .= "UPDATE \"" . $this->table . "\" SET ";
			$modif = "";
			foreach ($this->fields as $fieldName => $details)
			{
				if	(is_array($details) && $details["initial_value"] != $details["current_value"])
				{
					$request .= $modif;
					$request .= "\"$fieldName\" = ";
					$request .= $this->fieldToSQL($details["type"], $details["current_value"]);
					$modif = ", ";
				}
			}
			if	($modif == "")
				$request = "";
			else
			{
				$request .= " WHERE \"" . $this->identifierName . "\" = ";
				$request .= $this->fieldToSQL(
					$this->fields[$this->identifierName]["type"],
					$this->fields[$this->identifierName]["initial_value"]
				);
			}
		}
		elseif	($this->state == "n" && $this->modified)
		{
			$request .= "INSERT INTO \"" . $this->table . "\" (";
			$values = "";
			foreach ($this->fields as $fieldName => $details)
			{
				if	(is_array($details) && !is_null($details["current_value"]))
				{
					if	($values != "")
					{
						$request .= ", ";
						$values .= ", ";
					}
					$request .= "\"$fieldName\"";
					$values .= $this->fieldToSQL($details["type"], $details["current_value"]);
				}
				elseif	(is_array($details) && $details["mandatory"])
				{
					$this->errorMsg = "Missing mandatory fields";
					return	"";
				}
			}
			if	($values == "")
			{
				$request = "";
				$this->errorMsg = "No value set for this record";
			}
			else
				$request .= ") VALUES ($values)";
		}
		elseif	($this->state == "s" && $this->modified)
		{
			$request .= "DELETE FROM \"" . $this->table . "\"";
			$request .= " WHERE \"" . $this->identifierName . "\" = ";
			$request .= $this->fieldToSQL(
				$this->fields[$this->identifierName]["type"],
				$this->fields[$this->identifierName]["initial_value"]
			);
		}

		return	$request;
	}

	function	isModified()
	{
		return	$this->modified;
	}

	function	getError()
	{
		$err = $this->errorMsg;
		$this->errorMsg = "";
		return	$err;
	}

	function	isValid()
	{
		$this->state = "m";
		$this->modified = false;
		foreach ($this->fields as $name => $details)
		{
			$this->fields[$name]["initial_value"] = $this->fields[$name]["current_value"];
		}
	}

	function	getObjId()
	{
		return	$this->objId;
	}
}


class	d2p_enum_data extends d2p_data
{
	function	d2p_enum_data(&$manager, $identifier = NULL)
	{
		$this->d2p_data($manager, "enum", "id");

		if      (!is_null($identifier))
		{
			$db = $this->manager->getDatabaseConnection();
			$query = "SELECT * FROM \"" . $this->table . "\" WHERE \"" . $this->identifierName . "\" = ";
			$query .= $this->fieldToSQL(
				$this->fields[$this->identifierName]["type"], $identifier
			);
			$result = @pg_query($db, $query);
			if      (!$result)
			{
				$this->errorMsg = "Unable to execute query : $query";
				return  NULL;
			}
			$row = pg_fetch_array($result);
			$this->loadDatabaseEntry($row);
		}
	}

	function	prepareFields()
	{
		parent::prepareFields();
		$this->fields["id"] = array(
			"initial_value"	=> NULL,
			"current_value"	=> NULL,
			"type"		=> "int",
			"mandatory"	=> true
		);
		$this->fields["txt"] = array(
			"initial_value"	=> NULL,
			"current_value"	=> NULL,
			"type"		=> "text",
			"mandatory"	=> true
		);
	}

	function	destroy()
	{
		return	false;
	}

	function	setField()
	{
		return	false;
	}

	function	getHTMLForm()
	{
		return	"";
	}

	function	getDescription()
	{
		return $this->getField("txt");
	}
}


?>
